#include "mbed.h"
#include "GPS.h"
#include "MDM.h"

//----------------------------------------------------------------------
// You may need to configure these parameters

/** Set your secret SIM pin here "1234"
*/
#define SIMPIN      NULL

/** The APN of your network operator, sometimes it is "internet" 
    check your contract with the network operator
*/
#define APN         "gprs.swisscom.ch"

/** Set the user name for your APN, or NULL if not needed
*/
#define USERNAME    NULL

/** Set the password for your APN, or NULL if not needed
*/
#define PASSWORD    NULL 

//----------------------------------------------------------------------
/* This example was tested on C027-U20 and C027-G35 with the on board modem. 
   
   Additionally it was tested with a shield where the SARA-G350 RX/TX/PWRON 
   is connected to D0/D1/D4 and the GPS SCL/SDA is connected D15/D15. In this 
   configuration the following platforms were tested (it is likely that others 
   will work as well)
   - U-BLOX:    C027-XX.
   - NXP:       LPC1549v2 
   - Freescale: FRDM-KL25Z, FRDM-KL46Z
   - STM:       NUCLEO-F401RE, NUCLEO-F030R8
*/
#if defined(TARGET_UBLOX_C027) || defined(TARGET_LPC1768)
  #define C027_USEONBOARD // remove this if you have a GSM/GPS shield
  #ifdef C027_USEONBOARD
    #include "C027.h"
    C027 c027;
  #elif defined(TARGET_LPC1768) 
    // We assume we have a C027 (not an Arch Pro)
    #define D15  P0_1
    #define D14  P0_0
    #define D4   P2_12
    #define D1   P4_28
    #define D0   P4_29
    #define LED1 P3_25
  #endif
#endif

#ifndef C027_USEONBOARD
  #define GPSSCL    D15
  #define GPSSDA    D14
  #define GPSADR   (66<<1) // GPS I2C Address
  #define MDMPWRON  D4
  #define MDMTXD    D1
  #define MDMRXD    D0
  #define MDMCTS    NC
  #define MDMRTS    NC
  #define MDMBAUD   115200
#endif

// no tracing if serial is shared with the VCP
#define TRACE     ((USBRX==MDMRXD)||(USBTX==MDMTXD))?:printf

void printDeviceStatus(MDMParser::DevStatus* status) {
    TRACE("Device Status:\r\n");
    const char* txtDev[] = { "Unknown", "SARA-G350", "LISA-U200", "LISA-C200" };
    if (status->dev < sizeof(txtDev)/sizeof(*txtDev) && (status->dev != MDMParser::DEV_UNKNOWN))
        TRACE("  Device:       %s\r\n", txtDev[status->dev]);
    const char* txtLpm[] = { "Disabled", "Enabled", "Active" };
    if (status->lpm < sizeof(txtLpm)/sizeof(*txtLpm))
        TRACE("  Power Save:   %s\r\n", txtLpm[status->lpm]);
    const char* txtSim[] = { "Unknown", "Pin", "Ready" };
    if (status->sim < sizeof(txtSim)/sizeof(*txtSim) && (status->sim != MDMParser::SIM_UNKNOWN))
        TRACE("  SIM:          %s\r\n", txtSim[status->sim]);
    if (*status->ccid)  
        TRACE("  CCID:         %s\r\n", status->ccid);
    if (*status->imei) 
        TRACE("  IMEI:         %s\r\n", status->imei);
    if (*status->imsi)  
        TRACE("  IMSI:         %s\r\n", status->imsi);
    if (*status->meid) 
        TRACE("  MEID:         %s\r\n", status->meid);
    if (*status->manu) 
        TRACE("  Manufacturer: %s\r\n", status->manu);
    if (*status->model)  
        TRACE("  Model:        %s\r\n", status->model);
    if (*status->ver)  
        TRACE("  Version:      %s\r\n", status->ver);
}

void printNetStatus(MDMParser::NetStatus *status)
{
    TRACE("Network Status:\r\n");
    const char* txtReg[] = { "Unknown", "Denied", "None", "Home", "Roaming" };
    if (status->reg < sizeof(txtReg)/sizeof(*txtReg) && (status->reg != MDMParser::REG_UNKNOWN))
        TRACE("  Registration:       %s\r\n", txtReg[status->reg]);
    const char* txtAct[] = { "Unknown", "GSM", "Edge", "3G", "CDMA" };
    if (status->act < sizeof(txtAct)/sizeof(*txtAct) && (status->act != MDMParser::ACT_UNKNOWN))
        TRACE("  Access Technology:  %s\r\n", txtAct[status->act]);
    if (status->rssi) 
        TRACE("  Signal Strength:    %d dBm\r\n", status->rssi);
    if (*status->opr)  
        TRACE("  Operator:           %s\r\n", status->opr);
    if (*status->num)  
        TRACE("  Phone Number:       %s\r\n", status->num);
}

int main(void)
{
    int ret;
    char buf[512] = "";

    // only trace if 
    if ((USBRX!=MDMRXD)&&(USBTX!=MDMTXD)) {
        Serial pc(USBTX,USBRX);
        pc.baud(115200);
    }
    
    wait_ms(1000);
    
    TRACE("GSM/GPS Support Example\r\n");
    
#ifdef C027_USEONBOARD
    // turn on the supplies of the Modem and the GPS
    c027.mdmPower(true);
    c027.gpsPower(true);
#else
    // turn on the Modem
    DigitalOut mdmPwrOn(MDMPWRON, 0);
    wait_ms(150);
    mdmPwrOn = 1;
#endif
    wait(2);
    
    // Create the GPS object
#if defined(GPSSCL) && defined(GPSSDA) && defined(GPSADR)
    GPSI2C gps(GPSSDA,GPSSCL,GPSADR);     // use GPSI2C class
#elif defined(GPSTXD) && defined(GPSRXD) && defined(GPSBAUD)
    GPSSerial gps(GPSTXD,GPSRXD,GPSBAUD); // or GPSSerial class 
#else
    #warning "please define the pins for the GPS"
#endif
    
    // Create the modem object
    MDMSerial mdm(MDMTXD,MDMRXD,MDMBAUD
#if DEVICE_SERIAL_FC
                ,MDMRTS,MDMCTS
#endif
                );
    
    // initialize the modem 
    TRACE("Device Init\r\n");
    MDMParser::DevStatus devStatus;
    MDMParser::NetStatus netStatus;
    bool mdmOk = mdm.init(SIMPIN, &devStatus);
    if (mdmOk)
    {
        printDeviceStatus(&devStatus);
        
        // wait until we are connected
        TRACE("Network Check\r\n");
        while (!mdm.checkNetStatus(&netStatus))
            wait_ms(1000);
    
        printNetStatus(&netStatus);
        TRACE("Network Join\r\n");
        // join the internet connection 
        MDMParser::IP ip = 
            mdm.join(APN,USERNAME,PASSWORD);
        if (ip != NOIP)
        {
            TRACE("  IP Address: " IPSTR "\r\n", IPNUM(ip));
            TRACE("Socket Create\r\n");
            int socket = mdm.socketSocket(MDMParser::IPPROTO_TCP);
            if (socket >= 0)
            {
                TRACE("Socket Connect\r\n");
                if (mdm.socketConnect(socket, "mbed.org", 80))
                {
                    TRACE("Make a Http Post Request\r\n");
                    const char http[] = "GET /media/uploads/mbed_official/hello.txt HTTP/1.0\r\n\r\n";
                    TRACE("Socket Send\r\n");
                    mdm.socketSend(socket, http, sizeof(http)-1);
                
                    TRACE("Socket Recving\r\n");
                    while (true) {
                        ret = mdm.socketReadable(socket);
                        if (ret > 0)
                            ret = mdm.socketRecv(socket, buf, sizeof(buf)-1);
                        if (ret < 0)
                            break;
                        else if (ret > 0)
                            TRACE("Socket Recv \"%*s\"\r\n", ret, buf);
                    }
                    TRACE("Socket Close\r\n");
                    mdm.socketClose(socket);
                }
                TRACE("Socket Free\r\n");
                mdm.socketFree(socket);
            }
            
            // disconnect  
            TRACE("Network Disconnect\r\n");
            mdm.disconnect();
        }
        
        const char* ussd = "*#134#"; // You may get answer "UNKNOWN APPLICATION"
        TRACE("Send Ussd Command %s\r\n", ussd);
        ret = mdm.ussdCommand(ussd, buf);
        if (ret > 0) 
            TRACE("Got Ussd Answer: \"%*s\"\r\n", ret, buf);
    }
    TRACE("Checking SMS and GPS\r\n");
    char link[128] = "";
    unsigned int i = 0xFFFFFFFF;
    const int wait = 100;
    bool abort = false;
    //DigitalOut led(LED1);
    while (!abort) {
    //    led = !led;        
        while ((ret = gps.getMessage(buf, sizeof(buf))) > 0)
        {
            int len = LENGTH(ret);
            //TRACE("NMEA: %.*s\r\n", len-2, msg); 
            if ((PROTOCOL(ret) == NMEA) && (len > 6) && !strncmp("$GPGLL", buf, 6))
            {
                double la = 0, lo = 0;
                char ch;
                if (gps.getNmeaAngle(1,buf,len,la) && 
                    gps.getNmeaAngle(3,buf,len,lo) && 
                    gps.getNmeaItem(6,buf,len,ch) && ch == 'A')
                {
                    TRACE("GPS Location: %.5f %.5f\r\n", la, lo); 
                    sprintf(link, "I am here!\n"
                                  "https://maps.google.com/?q=%.5f,%.5f", la, lo); 
                }
            }
        }
        if (mdmOk && (i++ == 10000/wait)) {
            i = 0;
            // check the network status
            if (mdm.checkNetStatus(&netStatus))
                printNetStatus(&netStatus);
                
            // checking unread sms
            int ix[8];
            int n = mdm.smsList("REC UNREAD", ix, 8);
            if (8 < n) n = 8;
            while (0 < n--)
            {
                char num[32];
                TRACE("Unread SMS at index %d\r\n", ix[n]);
                if (mdm.smsRead(ix[n], num, buf, sizeof(buf))) {
                    TRACE("Got SMS from \"%s\" with text \"%s\"\r\n", num, buf);
                    TRACE("Delete SMS at index %d\r\n", ix[n]);
                    mdm.smsDelete(ix[n]);
                    // provide a reply
                    const char* reply = "Hello my friend";
                    if (strstr(buf, /*w*/"here are you"))
                        reply = *link ? link : "I don't know"; // reply wil location link
                    TRACE("Send SMS reply \"%s\" to \"%s\"\r\n", reply, num);
                    mdm.smsSend(num, reply);
                }
            }
        }
        wait_ms(wait);
    }
    mdm.powerOff();
    gps.powerOff();
    TRACE("Shutdown\r\n");
#ifdef C027_USEONBOARD
    // now it is safe to switch LDOs off
    c027.mdmPower(false);
    c027.gpsPower(false);
#endif    
    return 0;
}

